// Licensed to the .NET Foundation under one or more agreements.
// The .NET Foundation licenses this file to you under the MS-PL license.
// See the LICENSE file in the project root for more information.

using System;
using CoreGraphics;
using MvvmCross.Binding.Attributes;
using MvvmCross.Binding.BindingContext;
using UIKit;

namespace MvvmCross.Platforms.Tvos.Binding.Views
{
    public class MvxCollectionReusableView
		: UICollectionReusableView
		  , IMvxBindable
	{
		public IMvxBindingContext BindingContext { get; set; }

		public MvxCollectionReusableView()
		{
			this.CreateBindingContext();
		}

		public MvxCollectionReusableView(IntPtr handle)
			: base(handle)
		{
			this.CreateBindingContext();
		}

		public MvxCollectionReusableView(CGRect frame)
			: base(frame)
		{
			this.CreateBindingContext();
		}

		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				BindingContext.ClearAllBindings();
			}
			base.Dispose(disposing);
		}

		[MvxSetToNullAfterBinding]
		public object DataContext
		{
			get { return BindingContext.DataContext; }
			set { BindingContext.DataContext = value; }
		}
	}
}
